package com.keyware.shandan.system.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.keyware.shandan.common.util.StringUtils;
import com.keyware.shandan.system.entity.SysTheadConfig;
import com.keyware.shandan.system.mapper.SysTheadConfigMapper;
import com.keyware.shandan.system.service.SysTheadConfigService;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

/**
 * 前端数据表格表头配置数据库操作接口实现类
 *
 * @author GuoXin
 * @date 2023/8/30
 */
@Service
public class SysTheadConfigServiceImpl extends ServiceImpl<SysTheadConfigMapper, SysTheadConfig> implements SysTheadConfigService {
    static final String resourceTableId = "browser_resource_table", fileTableId = "browser_file_table";
    static List<SysTheadConfig> defaultConfig = new ArrayList<>();

    static {
        SysTheadConfig.SysTheadConfigBuilder builder = SysTheadConfig.builder();
        // 初始化综合浏览系统资源查询数据表表头默认配置
        defaultConfig.add(builder.tableId(resourceTableId).colName("resourceName").colTitle("资源名称").colSort(1).colWidth("25%").isShow(true).isSort(false).build());
        defaultConfig.add(builder.tableId(resourceTableId).colName("resourceComment").colTitle("资源注释").colSort(2).colWidth("200").isShow(false).isSort(false).build());
        defaultConfig.add(builder.tableId(resourceTableId).colName("directoryPath").colTitle("目录路径").colSort(3).colWidth("43%").isShow(true).isSort(false).build());
        defaultConfig.add(builder.tableId(resourceTableId).colName("themeTask").colTitle("主题任务").colSort(4).colWidth("200").isShow(false).isSort(false).build());
        defaultConfig.add(builder.tableId(resourceTableId).colName("dataSource").colTitle("数据来源").colSort(5).colWidth("100").isShow(false).isSort(false).build());
        defaultConfig.add(builder.tableId(resourceTableId).colName("taskTime").colTitle("任务时间").colSort(6).colWidth("160").isShow(true).isSort(false).build());
        defaultConfig.add(builder.tableId(resourceTableId).colName("dataType").colTitle("数据类型").colSort(7).colWidth("100").isShow(false).isSort(false).build());
        defaultConfig.add(builder.tableId(resourceTableId).colName("secretLevel").colTitle("数据密级").colSort(8).colWidth("100").isShow(true).isSort(false).build());
        defaultConfig.add(builder.tableId(resourceTableId).colName("resourceType").colTitle("资源类型").colSort(9).colWidth("100").isShow(true).isSort(false).build());
        // 初始化综合浏览系统文件全文检索数据表表头默认配置
        defaultConfig.add(builder.tableId(fileTableId).colName("fileName").colTitle("文件名称").colSort(1).colWidth("25%").isShow(true).isSort(false).build());
        defaultConfig.add(builder.tableId(fileTableId).colName("fileType").colTitle("文件类型").colSort(2).colWidth("100").isShow(true).isSort(false).build());
        defaultConfig.add(builder.tableId(fileTableId).colName("fileSize").colTitle("文件大小").colSort(3).colWidth("100").isShow(true).isSort(false).build());
        defaultConfig.add(builder.tableId(fileTableId).colName("remark").colTitle("文件描述").colSort(4).colWidth("300").isShow(false).isSort(false).build());
        defaultConfig.add(builder.tableId(fileTableId).colName("exerciseData").colTitle("是否演训数据").colSort(5).colWidth("120").isShow(true).isSort(false).build());
        defaultConfig.add(builder.tableId(fileTableId).colName("entryStaff").colTitle("录入人员").colSort(6).colWidth("100").isShow(true).isSort(false).build());
        defaultConfig.add(builder.tableId(fileTableId).colName("equipmentModel").colTitle("装备型号").colSort(7).colWidth("100").isShow(false).isSort(false).build());
        defaultConfig.add(builder.tableId(fileTableId).colName("inputDate").colTitle("任务时间").colSort(8).colWidth("100").isShow(true).isSort(false).build());
        defaultConfig.add(builder.tableId(fileTableId).colName("source").colTitle("文件来源").colSort(9).colWidth("100").isShow(false).isSort(false).build());
        defaultConfig.add(builder.tableId(fileTableId).colName("taskCode").colTitle("任务代号").colSort(10).colWidth("100").isShow(false).isSort(false).build());
        defaultConfig.add(builder.tableId(fileTableId).colName("taskNature").colTitle("任务性质").colSort(11).colWidth("100").isShow(false).isSort(false).build());
        defaultConfig.add(builder.tableId(fileTableId).colName("troopCode").colTitle("部队代号").colSort(12).colWidth("100").isShow(false).isSort(false).build());
        defaultConfig.add(builder.tableId(fileTableId).colName("targetNumber").colTitle("目标/靶标类型").colSort(13).colWidth("100").isShow(false).isSort(false).build());
        defaultConfig.add(builder.tableId(fileTableId).colName("missileNumber").colTitle("导弹编号").colSort(14).colWidth("100").isShow(false).isSort(false).build());
        defaultConfig.add(builder.tableId(fileTableId).colName("themeTask").colTitle("主题任务").colSort(15).colWidth("100").isShow(false).isSort(false).build());
        defaultConfig.add(builder.tableId(fileTableId).colName("secretLevel").colTitle("文件密级").colSort(16).colWidth("100").isShow(true).isSort(false).build());
        defaultConfig.add(builder.tableId(fileTableId).colName("text").colTitle("文件内容").colSort(17).colWidth("500").isShow(true).isSort(false).build());
    }

    @Override
    public List<SysTheadConfig> listByCreateUser(String userId) {
        if (StringUtils.hasText(userId)) {
            LambdaQueryWrapper<SysTheadConfig> query = new LambdaQueryWrapper<>();
            query.eq(SysTheadConfig::getCreateUser, userId);
            List<SysTheadConfig> list = list(query);
            if (list.isEmpty()) {
                // 为默认数据生成包含用户主键的ID
                defaultConfig.forEach(conf -> conf.genId(userId));
                return defaultConfig;
            }
            return list;
        }
        return Collections.emptyList();
    }

    @Override
    public boolean saveOrUpdateBatch(Collection<SysTheadConfig> entityList, int batchSize) {
        return super.saveOrUpdateBatch(entityList, batchSize);
    }
}
